package br.javali.editor.gui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import br.javali.editor.io.*;

/** 
* Constri uma janela com um mini editor de textos
* que l e salva em meio persistente (disco).
*/
public class MiniEditorFrame extends JFrame {
    
    // Elementos da GUI
    private JTextPane texto;
    private JButton[] botoes;
    private JMenuItem [] itens;
    
    // Rtulos: melhor definir externamente (i18n) - ResourceBundle
    private String labels[] = {"Arquivo", "Abrir...", "Gravar...", "Novo", "Sair"};
    private String comandos[] = {"open", "save", "new", "exit"};
    
    // Interface para meio de armazenamento
    private Persistencia repositorio;
    
    // Icones do Toolbar
    private Icon[] icones = {
        new ImageIcon(ClassLoader.getSystemResource("icons/open.gif")), 
        new ImageIcon(ClassLoader.getSystemResource("icons/save.gif")),
        new ImageIcon(ClassLoader.getSystemResource("icons/new.gif"))
    };
    
    /**
     * Classe interna que trata eventos de botes e menus
     */
    private ActionListener listener = new ActionListener() {
        public void actionPerformed(ActionEvent evt) {
            if (evt.getActionCommand().equals("exit")) {
                // if(arquivoSalvo() || confirmaPerdaDeDados())
                System.exit(0);
            } else if (evt.getActionCommand().equals("open")) {
                // if(arquivoSalvo() || confirmaPerdaDeDados())
                abreArquivo();
            } else if (evt.getActionCommand().equals("save")) {
                gravaArquivo();
            } else if (evt.getActionCommand().equals("new")) {
                // if(arquivoSalvo() || confirmaPerdaDeDados())
                texto.setText("");
            }
        }
    };

    /**
     * Construtor. nico membro pblico desta classe.
     * @param textoInicial Texto lido de arquivo opcionalmente passado em linha de comando
     */
    public MiniEditorFrame(String textoInicial) {
        super("MiniEditor 1.0");

        // Janela fecha e aplicao termina sem perguntar nada
        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        
        // obtm objeto que representa meio de armazenamento
        repositorio = Disco.getInstance();
        
        // obtm componentes da GUI
        JMenuBar mb = this.setupMenuBar();
        JToolBar tb = this.setupToolBar();
        texto = new JTextPane();
        texto.setText(textoInicial);
        
        // Monta componentes no ContentPane
        Container ct = this.getContentPane();
        ct.add(BorderLayout.NORTH, tb);
        ct.add(BorderLayout.CENTER, new JScrollPane(texto));
        this.setJMenuBar(mb);
        
        // Redimensiona e mostra a janela
        this.setSize(300,200);
        this.setVisible(true);
    }
    
    /**
     * Coloca os botes no ToolBar e o devolve.
     */
    private JToolBar setupToolBar() {
        JToolBar bar = new JToolBar();
        botoes = new JButton[comandos.length -1];
        for (int i = 0; i < botoes.length; i++) {
            botoes[i] = new JButton(icones[i]); 
            botoes[i].setActionCommand(comandos[i]);
            botoes[i].addActionListener(listener);
            botoes[i].setToolTipText(labels[i+1]); // labels tem um elemento a mais que comandos  
            bar.add(botoes[i]);
        }
        return bar;
    }
    
    /**
     * Coloca os menus no JMenuBar e o devolve.
     */
    private JMenuBar setupMenuBar() {
        JMenuBar mbar = new JMenuBar();
        itens = new JMenuItem[comandos.length];
        JMenu menu = new JMenu(labels[0]);
        for (int i = 0; i < itens.length; i++) {
            itens[i] = new JMenuItem(labels[i+1]); // labels tem um elemento a mais que comandos  
            itens[i].setActionCommand(comandos[i]);
            itens[i].addActionListener(listener);
            if (i == itens.length - 1) { // antes do exit, coloque um separador
                menu.addSeparator();
            }
            menu.add(itens[i]);
        }
        mbar.add(menu);
        return mbar;
    }
    
    /**
     * Carrega um arquivo do disco e coloca no JTextPane
     */
    private void abreArquivo() {
        JFileChooser chooser = new JFileChooser();
        int returnVal = chooser.showOpenDialog(this);
        if(returnVal == JFileChooser.APPROVE_OPTION) {
            String nomeArquivo = chooser.getSelectedFile().toString();
            try {
                String informacao = repositorio.load(nomeArquivo);
                if (informacao != null) {
                    texto.setText(informacao);
                }
            } catch (java.io.IOException e) {
                JOptionPane.showMessageDialog(this, "No foi possvel carregar " + nomeArquivo, "Erro", JOptionPane.ERROR_MESSAGE);       
            }
        }
    }
    
    /**
     * Grava o texto do TextPane em um arquivo.
     */
    private void gravaArquivo() {
        JFileChooser chooser = new JFileChooser();
        int returnVal = chooser.showSaveDialog(this);
        if(returnVal == JFileChooser.APPROVE_OPTION) {
            String nomeArquivo = chooser.getSelectedFile().toString();
            try {
                repositorio.save(texto.getText(), nomeArquivo);
            } catch (java.io.IOException e) {
                JOptionPane.showMessageDialog(this, "No foi possvel gravar " + nomeArquivo, "Erro", JOptionPane.ERROR_MESSAGE);  
            }
        }
    }
    
}
